﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace projectDoubleDeckerBus.Generics
{
    internal class SetGeneric<T>
    where T : class
    {
        /// <summary>
        /// Список объектов, которые храним
        /// </summary>
        private readonly List<T?> _places;
        /// <summary>
        /// Количество объектов в массиве
        /// </summary>
        /// Количество объектов в массиве
        /// </summary>
        public int Count => _places.Count;
        /// <summary>
        /// Максимальное количество объектов в списке
        /// </summary>
        private readonly int _maxCount;
        /// <summary>
        /// Конструктор
        /// </summary>
        /// <param name="count"></param>
        public SetGeneric(int count)
        {
            _maxCount = count;
            _places = new List<T?>(count);
        }
        /// </summary>
        /// <param name="bus">Добавляемый автобус</param>
        /// <returns></returns>
        public bool Insert(T bus)
        {
            return Insert(bus, 0);
        }
        public bool Insert(T bus, int position)
        {
            if (position < 0 || position >= _maxCount)
                return false;

            if (Count >= _maxCount)
                return false;
            _places.Insert(0, bus);
            return true;
        }
        public bool Remove(int position)
        {
            if (position < 0 || position > _maxCount)
                return false;
            if (position >= Count)
                return false;
            _places.RemoveAt(position);
            return true;
        }
        public T? this[int position]
        {
            get
            {
                if (position < 0 || position > _maxCount)
                    return null;
                return _places[position];
            }
            set
            {
                if (position < 0 || position > _maxCount)
                    return;
                _places[position] = value;
            }
        }
        public IEnumerable<T?> GetBuses(int? maxBuses = null)
        {
            for (int i = 0; i < _places.Count; ++i)
            {
                yield return _places[i];
                if (maxBuses.HasValue && i == maxBuses.Value)
                {
                    yield break;
                }
            }
        }

    }
}
